/*
 * rijndael-owf.c
 *
 * implementation of one-way function based on RIJNDAEL
 *
 * David A. McGrew
 * February, 2001
 */

/*
 *	
 * Copyright  (c) 2001, 2002, Cisco Systems, Inc.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials provided
 *   with the distribution.
 * 
 *   Neither the name of the Cisco Systems, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef RIJNDAEL_OWF
#define RIJNDAEL_OWF

#include "rijndael.h"

typedef struct {
  rijndael_expanded_key expanded_key;
} rijndael_owf_context;


/*  rijndael_owf_init initializes a rijndael_owf_context
 *
 *  the salt should be random (or at least be unique) in
 *  order to thwart precomputation attacks
 */

void
rijndael_owf_init(rijndael_owf_context *c, v128_t salt) {

  rijndael_expand_key(salt, c->expanded_key);
  
}

/*  rijndael_owf_evaluate evaluates the OWF at the value *x;
 *  the result occupies *x after the function returns
 */ 

void
rijndael_owf_evaluate(const rijndael_owf_context *c, v128_t *x) {
  v128_t tmp;

  /* store inital value */
  v128_copy(&tmp, x);

  /* encrypt inital value */
  rijndael_encrypt(x, c->expanded_key); 

  /* add initial value back into result of encryption */
  v128_xor(x, x, &tmp);
  
}

#endif /* RIJNDAEL_OWF */

